package com.platomix.api.cgi.controller;

import cn.hutool.json.JSONUtil;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.platomix.api.cgi.dto.AuthParamDto;
import com.platomix.api.cgi.entity.App;
import com.platomix.api.cgi.service.AppService;
import com.platomix.api.cgi.support.domain.TokenResult;
import com.platomix.api.cgi.support.properties.TokenProperties;
import com.platomix.api.cgi.support.util.AuthUtil;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.StringRedisTemplate;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.ResponseBody;
import org.springframework.web.bind.annotation.RestController;

import javax.validation.Valid;
import java.util.Objects;
import java.util.concurrent.TimeUnit;

/**
 * Created by IntelliJ IDEA.
 * Author: xingxin.jing
 * Date: 2019/3/24
 * Time: 下午4:44
 */
@Slf4j
@RestController
@RequestMapping("/oauth/2.0")
public class AuthController {

    @Autowired
    private AppService appService;

    @Autowired
    private TokenProperties tokenProperties;

    @Autowired
    private StringRedisTemplate stringRedisTemplate;

    /**
     * 获得token
     *
     * @param authParamDto
     * @return
     */
    @ResponseBody
    @RequestMapping("/token")
    public TokenResult auth(@RequestBody @Valid AuthParamDto authParamDto) {
        try {
            App app = appService.getOne(Wrappers.<App>lambdaQuery().eq(App::getApiKey, authParamDto.getApi_key()).eq(App::getSecretKey,authParamDto.getApi_secret()).last("limit 1"));
            if (Objects.isNull(app)) {
                return new TokenResult("invalid_api_key", "unknown api key");
            }
            //生成token
            String token = AuthUtil.getToken(app.getAppId(), app.getApiKey(), app.getSecretKey(), tokenProperties.getKey());
            Integer redisExpireSecs = tokenProperties.getExpireTime() * 24 * 3600;
            //token 放入redis，并设置失效时间
            stringRedisTemplate.opsForValue().set(token, app.getAppId().toString(), redisExpireSecs, TimeUnit.SECONDS);

            long expireTime = System.currentTimeMillis() / 1000 + redisExpireSecs;
            return new TokenResult(token, expireTime);
        } catch (Exception e) {
            log.error("生成token错误,param:" + JSONUtil.toJsonStr(authParamDto), e);
            return new TokenResult("invalid_api_key", "system_error");
        }
    }
}
